program matrix
  implicit none
  integer :: m_min, m_int, m_max 
  integer :: n_min, n_int, n_max 
  integer :: k_min, k_int, k_max 
  character(len=256) :: key
  real(8), allocatable :: d1(:,:), d2(:,:), d3(:,:)
  complex(8), allocatable :: z1(:,:), z2(:,:), z3(:,:)
  real(8) :: x,y

  call getarg(1,key); read(key,*)m_min
  call getarg(2,key); read(key,*)m_int
  call getarg(3,key); read(key,*)m_max
  call getarg(4,key); read(key,*)n_min
  call getarg(5,key); read(key,*)n_int
  call getarg(6,key); read(key,*)n_max
  call getarg(7,key); read(key,*)k_min
  call getarg(8,key); read(key,*)k_int
  call getarg(9,key); read(key,*)k_max

  call matrix_mult_d_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  call matrix_mult_s_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  call matrix_mult_h_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  call matrix_mult_z_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  call matrix_mult_c_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)

end

!-------------------------------------------------------------------------------
subroutine matrix_mult_d_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  implicit none
  integer, intent(in) :: m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max
  integer :: m, n, k
  real(8) :: flop, t1, t2, t3, t4

  do m = m_min, m_max, m_int
     do n = n_min, n_max, n_int
        do k = k_min, k_max, k_int
           call matrix_mult_d(m, n, k, t1, t2, t3, t4)
           flop=dble(m)*dble(n)*dble(2*k+2)
           write(0,300) m,n,k, &
                flop/t1 /dble(1000)**4, &
                flop/t2 /dble(1000)**4, &
                flop/t3 /dble(1000)**4, &
                flop/t4 /dble(1000)**4, &
                flop, t1, t2, t3, t4
        enddo
     enddo
  enddo
  
300  format (1x,"dgemm",i6,i6,i6,4(1x,f8.5),1x,e12.5 ,4(1x,e12.5))  
end 

!-------------------------------------------------------------------------------
subroutine matrix_mult_s_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  implicit none
  integer, intent(in) :: m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max
  integer :: m, n, k
  real(8) :: flop, t1, t2, t3, t4

  do m = m_min, m_max, m_int
     do n = n_min, n_max, n_int
        do k = k_min, k_max, k_int
           call matrix_mult_s(m, n, k, t1, t2, t3, t4)
           flop=dble(m)*dble(n)*dble(2*k+2)
           write(0,300) m,n,k, &
                flop/t1 /dble(1000)**4, &
                flop/t2 /dble(1000)**4, &
                flop/t3 /dble(1000)**4, &
                flop/t4 /dble(1000)**4, &
                flop, t1, t2, t3, t4
        enddo
     enddo
  enddo
  
300  format (1x,"sgemm",i6,i6,i6,4(1x,f8.5),1x,e12.5 ,4(1x,e12.5))  
end 

!-------------------------------------------------------------------------------
subroutine matrix_mult_h_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  implicit none
  integer, intent(in) :: m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max
  integer :: m, n, k
  real(8) :: flop, t1, t2, t3, t4

  do m = m_min, m_max, m_int
     do n = n_min, n_max, n_int
        do k = k_min, k_max, k_int
           call matrix_mult_h(m, n, k, t1, t2, t3, t4)
           flop=dble(m)*dble(n)*dble(2*k+2)
           write(0,300) m,n,k, &
                flop/t1 /dble(1000)**4, &
                flop/t2 /dble(1000)**4, &
                flop/t3 /dble(1000)**4, &
                flop/t4 /dble(1000)**4, &
                flop, t1, t2, t3, t4
        enddo
     enddo
  enddo
  
300  format (1x,"hgemm",i6,i6,i6,4(1x,f8.5),1x,e12.5 ,4(1x,e12.5))  
end 

!-------------------------------------------------------------------------------
subroutine matrix_mult_z_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  implicit none
  integer, intent(in) :: m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max
  integer :: m, n, k
  real(8) :: flop, t1, t2, t3, t4

  do m = m_min, m_max, m_int
     do n = n_min, n_max, n_int
        do k = k_min, k_max, k_int
           call matrix_mult_z(m, n, k, t1, t2, t3, t4)
           flop=dble(8)*dble(m)*dble(n)*dble(k) +dble(12)*dble(m)*dble(n)
           write(0,300) m,n,k, &
                flop/t1 /dble(1000)**4, &
                flop/t2 /dble(1000)**4, &
                flop/t3 /dble(1000)**4, &
                flop/t4 /dble(1000)**4, &
                flop, t1, t2, t3, t4
        enddo
     enddo
  enddo
  
300  format (1x,"zgemm",i6,i6,i6,4(1x,f8.5),1x,e12.5 ,4(1x,e12.5))  
end 

!-------------------------------------------------------------------------------
subroutine matrix_mult_c_all(m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max)
  implicit none
  integer, intent(in) :: m_min, m_int, m_max, n_min, n_int, n_max, k_min, k_int, k_max
  integer :: m, n, k
  real(8) :: flop, t1, t2, t3, t4

  do m = m_min, m_max, m_int
     do n = n_min, n_max, n_int
        do k = k_min, k_max, k_int
           call matrix_mult_c(m, n, k, t1, t2, t3, t4)
           flop=dble(8)*dble(m)*dble(n)*dble(k) +dble(12)*dble(m)*dble(n)
           write(0,300) m,n,k, &
                flop/t1 /dble(1000)**4, &
                flop/t2 /dble(1000)**4, &
                flop/t3 /dble(1000)**4, &
                flop/t4 /dble(1000)**4, &
                flop, t1, t2, t3, t4
        enddo
     enddo
  enddo
  
300  format (1x,"cgemm",i6,i6,i6,4(1x,f8.5),1x,e12.5 ,4(1x,e12.5))  
end 

!-------------------------------------------------------------------------------
subroutine matrix_mult_d(m,n,k,t1,t2,t3,t4)
  implicit none
  integer, intent(in) :: m, n, k
  real(8) :: a((m+1)*(k+1)), b((k+1)*(n+1)), c((m+1)*(n+1)), alpha, beta
  integer :: lda, ldb, ldc, ldat, ldbt
  real(8) :: t0, t1, t2, t3, t4
  real(8), external :: seconds

  alpha = 2.0
  beta  = 1.0
  ldc = m+1
  lda = m+1
  ldb = k+1
  ldat= k+1
  ldbt= n+1

  call dgemm("N", "N", M, N, K,  ALPHA,  A,  LDA,  B,  LDB, BETA, C, LDC )
  t0=seconds(); call dgemm("N", "N", M, N, K,  ALPHA,  A,  LDA ,  B,  LDB , BETA, C, LDC ); t1=seconds()-t0
  t0=seconds(); call dgemm("T", "N", M, N, K,  ALPHA,  A,  LDAT,  B,  LDB , BETA, C, LDC ); t2=seconds()-t0
  t0=seconds(); call dgemm("N", "T", M, N, K,  ALPHA,  A,  LDA ,  B,  LDBT, BETA, C, LDC ); t3=seconds()-t0
  t0=seconds(); call dgemm("T", "T", M, N, K,  ALPHA,  A,  LDAT,  B,  LDBT, BETA, C, LDC ); t4=seconds()-t0

end subroutine matrix_mult_d

!-------------------------------------------------------------------------------
subroutine matrix_mult_s(m,n,k,t1,t2,t3,t4)
  implicit none
  integer, intent(in) :: m, n, k
  real(4) :: a((m+1)*(k+1)), b((k+1)*(n+1)), c((m+1)*(n+1)), alpha, beta
  integer :: lda, ldb, ldc, ldat, ldbt
  real(8) :: t0, t1, t2, t3, t4
  real(8), external :: seconds

  alpha = 2.0
  beta  = 1.0
  ldc = m+1
  lda = m+1
  ldb = k+1
  ldat= k+1
  ldbt= n+1

  call sgemm("N", "N", M, N, K,  ALPHA,  A,  LDA,  B,  LDB, BETA, C, LDC )
  t0=seconds(); call sgemm("N", "N", M, N, K,  ALPHA,  A,  LDA ,  B,  LDB , BETA, C, LDC ); t1=seconds()-t0
  t0=seconds(); call sgemm("T", "N", M, N, K,  ALPHA,  A,  LDAT,  B,  LDB , BETA, C, LDC ); t2=seconds()-t0
  t0=seconds(); call sgemm("N", "T", M, N, K,  ALPHA,  A,  LDA ,  B,  LDBT, BETA, C, LDC ); t3=seconds()-t0
  t0=seconds(); call sgemm("T", "T", M, N, K,  ALPHA,  A,  LDAT,  B,  LDBT, BETA, C, LDC ); t4=seconds()-t0

end subroutine matrix_mult_s

!-------------------------------------------------------------------------------
subroutine matrix_mult_h(m,n,k,t1,t2,t3,t4)
  implicit none
  integer, intent(in) :: m, n, k
  real(2) :: a((m+1)*(k+1)), b((k+1)*(n+1)), c((m+1)*(n+1)), alpha, beta
  integer :: lda, ldb, ldc, ldat, ldbt
  real(8) :: t0, t1, t2, t3, t4
  real(8), external :: seconds

  alpha = 2.0
  beta  = 1.0
  ldc = m+1
  lda = m+1
  ldb = k+1
  ldat= k+1
  ldbt= n+1

  call fjblas_gemm_r16("N", "N", M, N, K,  ALPHA,  A,  LDA,  B,  LDB, BETA, C, LDC )
  t0=seconds(); call fjblas_gemm_r16("N", "N", M, N, K,  ALPHA,  A,  LDA ,  B,  LDB , BETA, C, LDC ); t1=seconds()-t0
  t0=seconds(); call fjblas_gemm_r16("T", "N", M, N, K,  ALPHA,  A,  LDAT,  B,  LDB , BETA, C, LDC ); t2=seconds()-t0
  t0=seconds(); call fjblas_gemm_r16("N", "T", M, N, K,  ALPHA,  A,  LDA ,  B,  LDBT, BETA, C, LDC ); t3=seconds()-t0
  t0=seconds(); call fjblas_gemm_r16("T", "T", M, N, K,  ALPHA,  A,  LDAT,  B,  LDBT, BETA, C, LDC ); t4=seconds()-t0

end subroutine matrix_mult_h

!-------------------------------------------------------------------------------
subroutine matrix_mult_z(m,n,k,t1,t2,t3,t4)
  implicit none
  integer, intent(in) :: m, n, k
  complex(8) :: a((m+1)*(k+1)), b((k+1)*(n+1)), c((m+1)*(n+1)), alpha, beta
  integer :: lda, ldb, ldc, ldat, ldbt
  real(8) :: t0, t1, t2, t3, t4
  real(8), external :: seconds

  alpha = 2.0
  beta  = 1.0
  ldc = m+1
  lda = m+1
  ldb = k+1
  ldat= k+1
  ldbt= n+1

  call zgemm("N", "N", M, N, K,  ALPHA,  A,  LDA,  B,  LDB, BETA, C, LDC )
  t0=seconds(); call zgemm("N", "N", M, N, K,  ALPHA,  A,  LDA ,  B,  LDB , BETA, C, LDC ); t1=seconds()-t0
  t0=seconds(); call zgemm("T", "N", M, N, K,  ALPHA,  A,  LDAT,  B,  LDB , BETA, C, LDC ); t2=seconds()-t0
  t0=seconds(); call zgemm("N", "T", M, N, K,  ALPHA,  A,  LDA ,  B,  LDBT, BETA, C, LDC ); t3=seconds()-t0
  t0=seconds(); call zgemm("T", "T", M, N, K,  ALPHA,  A,  LDAT,  B,  LDBT, BETA, C, LDC ); t4=seconds()-t0

end subroutine matrix_mult_z

!-------------------------------------------------------------------------------
subroutine matrix_mult_c(m,n,k,t1,t2,t3,t4)
  implicit none
  integer, intent(in) :: m, n, k
  complex(4) :: a((m+1)*(k+1)), b((k+1)*(n+1)), c((m+1)*(n+1)), alpha, beta
  integer :: lda, ldb, ldc, ldat, ldbt
  real(8) :: t0, t1, t2, t3, t4
  real(8), external :: seconds

  alpha = 2.0
  beta  = 1.0
  ldc = m+1
  lda = m+1
  ldb = k+1
  ldat= k+1
  ldbt= n+1

  call cgemm("N", "N", M, N, K,  ALPHA,  A,  LDA,  B,  LDB, BETA, C, LDC )
  t0=seconds(); call cgemm("N", "N", M, N, K,  ALPHA,  A,  LDA ,  B,  LDB , BETA, C, LDC ); t1=seconds()-t0
  t0=seconds(); call cgemm("T", "N", M, N, K,  ALPHA,  A,  LDAT,  B,  LDB , BETA, C, LDC ); t2=seconds()-t0
  t0=seconds(); call cgemm("N", "T", M, N, K,  ALPHA,  A,  LDA ,  B,  LDBT, BETA, C, LDC ); t3=seconds()-t0
  t0=seconds(); call cgemm("T", "T", M, N, K,  ALPHA,  A,  LDAT,  B,  LDBT, BETA, C, LDC ); t4=seconds()-t0

end subroutine matrix_mult_c

!-------------------------------------------------------------------------------
real(8) function seconds() ! returns wall clock seconds
  implicit none
  real(8)       :: count_to_seconds
  integer(8)    :: count, count_rate
  logical, save :: initialized = .false.
  if (.not. initialized) then
     call system_clock(count, count_rate)
     count_to_seconds = 1.0_8 / real(count_rate, kind = 8)
  endif
  call system_clock(count)
  seconds = count * count_to_seconds
end

