program matrix
  implicit none
  integer :: n_min, n_int, n_max 
  character(len=256) :: key


  call getarg(1,key); read(key,*)n_min
  call getarg(2,key); read(key,*)n_int
  call getarg(3,key); read(key,*)n_max

!  call matrix_inv_d_all(n_min, n_int, n_max)
!  call matrix_inv_s_all(n_min, n_int, n_max)
!  call matrix_inv_z_all(n_min, n_int, n_max)
  call matrix_inv_c_all(n_min, n_int, n_max)

end

!-------------------------------------------------------------------------------
subroutine matrix_inv_d_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x
  real(8) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1

  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        mat(i,j)=x
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_inv_d(mat, n, t1)
!     flop = 2 * dble(n)*dble(n)*dble(n)
     flop=1* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3 - 1) + dble(n)) + 2*dble(n)/3) + &  !getrt mul
          1* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3    ) - dble(n)) +   dble(n)/6) + &  !getrt add
          1*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3+0.5) ) + & ! getri mul
          1*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3-1.5) )     ! getri add
     write(0,300) n, flop/t1 /dble(1000)**4, flop, t1
  enddo
  
300  format (1x,"matrix_inv_d : ",i6, e12.5, e12.5, e12.5)
end 

!-------------------------------------------------------------------------------
subroutine matrix_inv_s_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x
  real(4) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1

  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        mat(i,j)=x
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_inv_s(mat, n, t1)
     flop=1* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3 - 1) + dble(n)) + 2*dble(n)/3) + &  !getrt mul
          1* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3    ) - dble(n)) +   dble(n)/6) + &  !getrt add
          1*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3+0.5) ) + & ! getri mul
          1*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3-1.5) )     ! getri add
     write(0,300) n, flop/t1 /dble(1000)**4, flop, t1
  enddo
  
300  format (1x,"matrix_inv_s : ",i6, e12.5, e12.5, e12.5)
end 

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
subroutine matrix_inv_z_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x,y
  complex(8) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1

  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        call random_number(y)
        mat(i,j)=dcmplx(x,y)
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_inv_z(mat, n, t1)
!     flop = 8 * dble(n)*dble(n)*dble(n)
     flop=6* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3 - 1) + dble(n)) + 2*dble(n)/3) + &  !getrt mul
          2* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3    ) - dble(n)) +   dble(n)/6) + &  !getrt add
          6*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3+0.5) ) + & ! getri mul
          2*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3-1.5) )     ! getri add
     write(0,300) n, flop/t1 /dble(1000)**4, flop, t1
  enddo
  
300  format (1x,"matrix_inv_z : ",i6, e12.5, e12.5, e12.5)
end 

!-------------------------------------------------------------------------------
subroutine matrix_inv_c_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x,y
  complex(4) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1

  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        call random_number(y)
        mat(i,j)=dcmplx(x,y)
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_inv_c(mat, n, t1)
!     flop = 8 * dble(n)*dble(n)*dble(n)
     flop=6* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3 - 1) + dble(n)) + 2*dble(n)/3) + &  !getrt mul
          2* (dble(n)/2 * (dble(n) * (dble(n) - dble(n)/3    ) - dble(n)) +   dble(n)/6) + &  !getrt add
          6*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3+0.5) ) + & ! getri mul
          2*  dble(n) * ( 5/6 + dble(n) * (2*dble(n)/3-1.5) )     ! getri add
     write(0,300) n, flop/t1 /dble(1000)**4, flop, t1
  enddo
  
300  format (1x,"matrix_inv_c : ",i6, e12.5, e12.5, e12.5)
end 

!-------------------------------------------------------------------------------
subroutine matrix_inv_d(mat, n, time1)
  implicit none
  integer, intent(in) :: n
  real(8) :: mat(n,n), tw
  real(8), allocatable :: work(:), iwork(:)
  integer :: ipiv(n), i, lwork, liwork, lda_x, lda_y
  real(8) :: time0, time1
  real(8), external :: seconds

  call dgetrf(n, n, mat, n, ipiv, i)
  call dgetri(n, mat, n, ipiv, tw, -1 ,i)
  lwork=nint(tw)
  allocate(work(lwork+1))
  work=0d0
  call dgetri(n, mat, n, ipiv, work, n ,i)

  time0=seconds()
  call dgetrf(n, n, mat, n, ipiv, i)
  call dgetri(n, mat, n, ipiv, work, n ,i)
  time1=seconds()-time0
  deallocate(work)
  
end

!-------------------------------------------------------------------------------
subroutine matrix_inv_s(mat, n, time1)
  implicit none
  integer, intent(in) :: n
  real(4) :: mat(n,n), tw
  real(4), allocatable :: work(:), iwork(:)
  integer :: ipiv(n), i, lwork, liwork, lda_x, lda_y
  real(8) :: time0, time1, flop
  real(8), external :: seconds

  call sgetrf(n, n, mat, n, ipiv, i)
  call sgetri(n, mat, n, ipiv, tw, -1 ,i)
  lwork=nint(tw)
  allocate(work(lwork+1))
  work=0d0
  call sgetri(n, mat, n, ipiv, work, n ,i)

  time0=seconds()
  call sgetrf(n, n, mat, n, ipiv, i)
  call sgetri(n, mat, n, ipiv, work, n ,i)
  time1=seconds()-time0
  deallocate(work)
  
end
!-------------------------------------------------------------------------------

subroutine matrix_inv_z(mat, n, time1)
  implicit none
  integer, intent(in) :: n
  complex(8) :: mat(n,n), tw
  complex(8), allocatable :: work(:), iwork(:)
  integer :: ipiv(n), i, lwork, liwork, lda_x, lda_y
  real(8) :: time0, time1, flop
  real(8), external :: seconds

  call zgetrf(n, n, mat, n, ipiv, i)
  call zgetri(n, mat, n, ipiv, tw, -1 ,i)
  lwork=nint(dble(tw))
  allocate(work(lwork+1))
  work=dcmplx(0d0,0d0)
  call zgetri(n, mat, n, ipiv, work, n ,i)

  
  time0=seconds()
  call zgetrf(n, n, mat, n, ipiv, i)
  call zgetri(n, mat, n, ipiv, work, n ,i)
  time1=seconds()-time0
  deallocate(work)
  
end

!-------------------------------------------------------------------------------
subroutine matrix_inv_c(mat, n, time1)
  implicit none
  integer, intent(in) :: n
  complex(4) :: mat(n,n), tw
  complex(4), allocatable :: work(:), iwork(:)
  integer :: ipiv(n), i, lwork, liwork, lda_x, lda_y
  real(8) :: time0, time1
  real(8), external :: seconds

  call cgetrf(n, n, mat, n, ipiv, i)
  call cgetri(n, mat, n, ipiv, tw, -1 ,i)
  lwork=nint(dble(tw))
  allocate(work(lwork+1))
  work=dcmplx(0d0,0d0)
  call cgetri(n, mat, n, ipiv, work, n ,i)
  
  time0=seconds()
  call cgetrf(n, n, mat, n, ipiv, i)
  call cgetri(n, mat, n, ipiv, work, n ,i)
  time1=seconds()-time0
  deallocate(work)

end

!-------------------------------------------------------------------------------
real(8) function seconds() ! returns wall clock seconds
  implicit none
  real(8)       :: count_to_seconds
  integer(8)    :: count, count_rate
  logical, save :: initialized = .false.
  if (.not. initialized) then
     call system_clock(count, count_rate)
     count_to_seconds = 1.0_8 / real(count_rate, kind = 8)
  endif
  call system_clock(count)
  seconds = count * count_to_seconds
end

