program matrix
  implicit none
  integer :: n_min, n_int, n_max 
  character(len=256) :: key

write(*,*)"FLOP may be incorrect"
  call getarg(1,key); read(key,*)n_min
  call getarg(2,key); read(key,*)n_int
  call getarg(3,key); read(key,*)n_max

  call matrix_svd_d_all(n_min, n_int, n_max)
!  call matrix_svd_s_all(n_min, n_int, n_max)
!  call matrix_svd_z_all(n_min, n_int, n_max)
!  call matrix_svd_c_all(n_min, n_int, n_max)

end

!-------------------------------------------------------------------------------
subroutine matrix_svd_d_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x
  real(8) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1, t2

  !$omp parallel do private(i, j, x)
  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        mat(i,j)=x
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_svd_d(mat, n, t1, t2)
     flop = dble(20)/dble(3) * dble(n)*dble(n)*dble(n)
     write(0,300) n, &
          flop/t1 /dble(1000)**4, &
          flop/t2 /dble(1000)**4, &
          flop, t1, t2
  enddo
  
300  format (1x,"matrix_svd_d : ",i6, 2e12.5, e12.5, 2e12.5)
end 

!-------------------------------------------------------------------------------
subroutine matrix_svd_s_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x
  real(4) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1, t2

  !$omp parallel do private(i, j, x)
  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        mat(i,j)=x
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_svd_s(mat, n, t1, t2)
     flop = dble(20)/dble(3) * dble(n)*dble(n)*dble(n)
     write(0,300) n, &
          flop/t1 /dble(1000)**4, &
          flop/t2 /dble(1000)**4, &
          flop, t1, t2
  enddo
  
300  format (1x,"matrix_svd_s : ",i6, 2e12.5, e12.5, 2e12.5)
end 

!-------------------------------------------------------------------------------
subroutine matrix_svd_z_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x, y
  complex(8) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1, t2

  !$omp parallel do private(i, j, x, y)
  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        call random_number(y)
        mat(i,j)=dcmplx(x,y)
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_svd_z(mat, n, t1, t2)
     flop = dble(20)/dble(3) * dble(n)*dble(n)*dble(n)
     write(0,300) n, &
          flop/t1 /dble(1000)**4, &
          flop/t2 /dble(1000)**4, &
          flop, t1, t2
  enddo
  
300  format (1x,"matrix_svd_z : ",i6, 2e12.5, e12.5, 2e12.5)
end 

!-------------------------------------------------------------------------------
subroutine matrix_svd_c_all(n_min, n_int, n_max)
  implicit none
  integer, intent(in) :: n_min, n_int, n_max
  real(8) :: x, y
  complex(4) :: mat(n_max,n_max)
  integer :: n, i, j
  real(8) :: flop, t1, t2

  !$omp parallel do private(i, j, x)
  do i= 1, n_max
     do j= 1, n_max
        call random_number(x)
        call random_number(y)
        mat(i,j)=dcmplx(x,y)
     enddo
  enddo
  
  do n = n_min, n_max, n_int
     call matrix_svd_c(mat, n, t1, t2)
     flop = dble(20)/dble(3) * dble(n)*dble(n)*dble(n)
     write(0,300) n, &
          flop/t1 /dble(1000)**4, &
          flop/t2 /dble(1000)**4, &
          flop, t1, t2
  enddo
  
300  format (1x,"matrix_svd_c : ",i6, 2e12.5, e12.5, 2e12.5)
end 

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
subroutine matrix_svd_d(mat, n, time1, time2)
  implicit none
  !                           2147483647
  integer, parameter :: lwork=1000000000
  integer, intent(in) :: n
  real(8) :: mat(n,n), u(n,n), v(n,n), l(n)
  real(8) :: work(lwork)
  integer :: i, iwork(8*n)
  real(8) :: time0, time1, time2
  real(8), external :: seconds

  call dgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, -1, i)
  time0=seconds()
  call dgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, nint(work(1)), i)
  time1=seconds()-time0

  call dgesdd('A', n, n, mat, n, L, U, n, V, n, work, -1, iwork, i)
  time0=seconds()
  call dgesdd('A', n, n, mat, n, L, U, n, V, n, work, nint(work(1)), iwork, i)
  time2=seconds()-time0

end

!-------------------------------------------------------------------------------
subroutine matrix_svd_s(mat, n, time1, time2)
  implicit none
  !                           2147483647
  integer, parameter :: lwork=1000000000
  integer, intent(in) :: n
  real(4) :: mat(n,n), u(n,n), v(n,n), l(n)
  real(4) :: work(lwork)
  integer :: i, iwork(8*n)
  real(8) :: time0, time1, time2
  real(8), external :: seconds

  call sgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, -1, i)
  time0=seconds()
  call sgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, nint(work(1)), i)
  time1=seconds()-time0

  call sgesdd('A', n, n, mat, n, L, U, n, V, n, work, -1, iwork, i)
  time0=seconds()
  call sgesdd('A', n, n, mat, n, L, U, n, V, n, work, nint(work(1)), iwork, i)
  time2=seconds()-time0

end

!-------------------------------------------------------------------------------
subroutine matrix_svd_z(mat, n, time1, time2)
  implicit none
  !                           2147483647
  integer, parameter :: lwork=1000000000
  integer, intent(in) :: n
  complex(8) :: mat(n,n), u(n,n), v(n,n), work(lwork)
  real(8) :: rwork( (5*n+7)*n ), l(n)
  integer :: i, iwork(8*n)
  real(8) :: time0, time1, time2
  real(8), external :: seconds

  call zgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, -1, rwork, i)
  time0=seconds()
  call zgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, nint(dble(work(1))), rwork, i)
  time1=seconds()-time0

  call zgesdd('A', n, n, mat, n, L, U, n, V, n, work, -1, rwork, iwork, i)
  time0=seconds()
  call zgesdd('A', n, n, mat, n, L, U, n, V, n, work, nint(dble(work(1))), rwork, iwork, i)
  time2=seconds()-time0

end

!-------------------------------------------------------------------------------
subroutine matrix_svd_c(mat, n, time1, time2)
  implicit none
  !                           2147483647
  integer, parameter :: lwork=1000000000
  integer, intent(in) :: n
  complex(4) :: mat(n,n), u(n,n), v(n,n), work(lwork)
  real(4) :: rwork( (5*n+7)*n ), l(n)
  integer :: i, iwork(8*n)
  real(8) :: time0, time1, time2
  real(8), external :: seconds

  call cgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, -1, rwork, i)
  time0=seconds()
  call cgesvd('A','A', n, n, mat, n, L, U, n, V, n, work, nint(dble(work(1))), rwork, i)
  time1=seconds()-time0

  call cgesdd('A', n, n, mat, n, L, U, n, V, n, work, -1, rwork, iwork, i)
  time0=seconds()
  call cgesdd('A', n, n, mat, n, L, U, n, V, n, work, nint(dble(work(1))), rwork, iwork, i)
  time2=seconds()-time0

end

!-------------------------------------------------------------------------------
real(8) function seconds() ! returns wall clock seconds
  implicit none
  real(8)       :: count_to_seconds
  integer(8)    :: count, count_rate
  logical, save :: initialized = .false.
  if (.not. initialized) then
     call system_clock(count, count_rate)
     count_to_seconds = 1.0_8 / real(count_rate, kind = 8)
  endif
  call system_clock(count)
  seconds = count * count_to_seconds
end

